==> pyPhases-1.1.20.tar.gz: pyPhases-1.1.20/pyPhases/util/pdict.py <==
class pdict(dict):
    def __getitem__(self, *args, create=False):
        k = args[0] if len(args) == 1 else args
        if not isinstance(k, list) and not isinstance(k, tuple):
            return super().__getitem__(k)

        value = self
        for field in k:
            try:
                if isinstance(value, list):
                    field = int(field)
                value = value.__getitem__(field)
            except KeyError as e:
                if create:
                    v = pdict({})
                    value.__setitem__(field, v)
                    value = value.__getitem__(field)
                else:
                    raise e

        return value

    def __setitem__(self, orgPath, v):
        if not isinstance(orgPath, list) and not isinstance(orgPath, tuple):
            return super().__setitem__(orgPath, v)
        orgPath = list(orgPath)
        value = self
        k = orgPath.copy()
        overwriteField = k.pop()
        for field in k:
            try:
                value = value.__getitem__(field)
            except KeyError:
                value.__setitem__(field, pdict())
                value = value.__getitem__(field)
        value[overwriteField] = v

    def setdefaults(self, defaultDict):
        for key, value in pdict(defaultDict).items():
            if key in self:
                if isinstance(self[key], dict) and bool(value):
                    self[key] = pdict(self[key])
                    self[key].setdefaults(value)
            else:
                self[key] = value
        return self

    def update(self, values, path=None):
        path = path or []
        for field in values:
            value = values[field]

            if isinstance(value, dict) and field in self:
                self[field] = pdict(self[field])
                self[field].update(value)
            else:
                self[field] = value
        pass

==> pyPhases-1.2.1.tar.gz: pyPhases-1.2.1/pyPhases/util/pdict.py <==
class pdict(dict):
    def __getitem__(self, *args, create=False):
        k = args[0] if len(args) == 1 else args
        if not isinstance(k, list) and not isinstance(k, tuple):
            return super().__getitem__(k)

        value = self
        for field in k:
            try:
                if isinstance(value, list):
                    field = int(field)
                value = value.__getitem__(field)
            except KeyError as e:
                if not create:
                    raise e

                v = pdict({})
                value.__setitem__(field, v)
                value = value.__getitem__(field)
        return value

    def __setitem__(self, orgPath, v):
        if not isinstance(orgPath, list) and not isinstance(orgPath, tuple):
            return super().__setitem__(orgPath, v)

        orgPath = list(orgPath)
        parentPath = orgPath[:-1]
        overwriteField = orgPath[-1]
        parent = self[parentPath]

        if isinstance(parent, list):
            overwriteField = int(overwriteField)
        parent[overwriteField] = v

    def setdefaults(self, defaultDict):
        for key, value in pdict(defaultDict).items():
            if key in self:
                if isinstance(self[key], dict) and bool(value):
                    self[key] = pdict(self[key])
                    self[key].setdefaults(value)
            else:
                self[key] = value

    def update(self, values, path=None):
        path = path or []
        for field in values:
            value = values[field]

            if isinstance(value, dict) and field in self:
                self[field] = pdict(self[field])
                self[field].update(value)
            else:
                self[field] = value
